@@ -2,16 +2,26 @@ from DateTime import DateTime
 from Globals import InitializeClass
 from AccessControl import ClassSecurityInfo
 
+from zope.interface import implements
+
 from Products.CMFCore.utils import getToolByName
 
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
 
 from Products.ResourceRegistries import config
 from Products.ResourceRegistries import permissions
+from Products.ResourceRegistries.interfaces.ResourceRegistries import ICSSRegistry as z2ICSSRegistry
 from Products.ResourceRegistries.interfaces import ICSSRegistry
 from Products.ResourceRegistries.tools.BaseRegistry import BaseRegistryTool
 from Products.ResourceRegistries.tools.BaseRegistry import Resource
 
+import re
+from packer import CSSPacker
+
+
+csspacker = CSSPacker('safe')
+csspacker_full = CSSPacker('full')
+
 
 class Stylesheet(Resource):
     security = ClassSecurityInfo()
@@ -22,6 +32,7 @@ class Stylesheet(Resource):
         self._data['rel'] = kwargs.get('rel', 'stylesheet')
         self._data['title'] = kwargs.get('title', '')
         self._data['rendering'] = kwargs.get('rendering', 'import')
+        self._data['compression'] = kwargs.get('compression', 'safe')
 
     security.declarePublic('getMedia')
     def getMedia(self):
@@ -61,6 +72,19 @@ class Stylesheet(Resource):
     def setRendering(self, rendering):
         self._data['rendering'] = rendering
 
+    security.declarePublic('getCompression')
+    def getCompression(self):
+        # as this is a new property, old instance might not have that value, so
+        # return 'safe' as default
+        compression = self._data.get('compression', 'safe')
+        if compression in config.CSS_COMPRESSION_METHODS:
+            return compression
+        return 'none'
+
+    security.declareProtected(permissions.ManagePortal, 'setCompression')
+    def setCompression(self, compression):
+        self._data['compression'] = compression
+
 InitializeClass(Stylesheet)
 
 
@@ -73,7 +97,8 @@ class CSSRegistryTool(BaseRegistryTool):
 
     security = ClassSecurityInfo()
 
-    __implements__ = (BaseRegistryTool.__implements__, ICSSRegistry,)
+    implements(ICSSRegistry)
+    __implements__ = (BaseRegistryTool.__implements__, z2ICSSRegistry,)
 
     #
     # ZMI stuff
@@ -139,12 +164,27 @@ class CSSRegistryTool(BaseRegistryTool):
         if previtem.getMedia() != resource.getMedia():
             previtem.setMedia(None)
 
+    def _compressCSS(self, content, level='safe'):
+        if level == 'full':
+            return csspacker_full.pack(content)
+        elif level == 'safe':
+            return csspacker.pack(content)
+        else:
+            return content
+
     security.declarePrivate('finalizeContent')
     def finalizeContent(self, resource, content):
         """Finalize the resource content."""
+        compression = resource.getCompression()
+        if compression != 'none' and not self.getDebugMode():
+            orig_url = "%s/%s?original=1" % (self.absolute_url(), resource.getId())
+            content = "/* %s */\n%s" % (orig_url,
+                                     self._compressCSS(content, compression))
+
         m = resource.getMedia()
         if m:
-            return '@media %s {\n%s\n}\n' % (m, content)
+            content = '@media %s {\n%s\n}\n' % (m, content)
+
         return content
 
     #
@@ -154,10 +194,11 @@ class CSSRegistryTool(BaseRegistryTool):
     security.declareProtected(permissions.ManagePortal, 'manage_addStylesheet')
     def manage_addStylesheet(self, id, expression='', media='',
                              rel='stylesheet', title='', rendering='import',
-                             enabled=False, cookable=True, REQUEST=None):
+                             enabled=False, cookable=True, compression='safe',
+                             cacheable=True, REQUEST=None):
         """Register a stylesheet from a TTW request."""
         self.registerStylesheet(id, expression, media, rel, title,
-                                rendering, enabled, cookable)
+                                rendering, enabled, cookable, compression, cacheable)
         if REQUEST:
             REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])
 
@@ -182,7 +223,8 @@ class CSSRegistryTool(BaseRegistryTool):
                                     rendering=r.get('rendering', 'import'),
                                     enabled=r.get('enabled', False),
                                     cookable=r.get('cookable', False),
-                                    cacheable=r.get('cacheable', False))
+                                    cacheable=r.get('cacheable', False),
+                                    compression=r.get('compression', 'safe'))
             stylesheets.append(stylesheet)
         self.resources = tuple(stylesheets)
         self.cookResources()
@@ -202,7 +244,8 @@ class CSSRegistryTool(BaseRegistryTool):
 
     security.declareProtected(permissions.ManagePortal, 'registerStylesheet')
     def registerStylesheet(self, id, expression='', media='', rel='stylesheet',
-                           title='', rendering='import',  enabled=1, cookable=True):
+                           title='', rendering='import',  enabled=1,
+                           cookable=True, compression='safe', cacheable=True):
         """Register a stylesheet."""
         stylesheet = Stylesheet(id,
                                 expression=expression,
@@ -211,14 +254,46 @@ class CSSRegistryTool(BaseRegistryTool):
                                 title=title,
                                 rendering=rendering,
                                 enabled=enabled,
-                                cookable=cookable)
+                                cookable=cookable,
+                                compression=compression,
+                                cacheable=cacheable)
         self.storeResource(stylesheet)
 
+    security.declareProtected(permissions.ManagePortal, 'updateStylesheet')
+    def updateStylesheet(self, id, **data):
+        stylesheet = self.getResourcesDict().get(id, None)
+        if stylesheet is None:
+            raise ValueError, 'Invalid resource id %s' % (id)
+        
+        if data.get('expression', None) is not None:
+            stylesheet.setExpression(data['expression'])
+        if data.get('media', None) is not None:
+            stylesheet.setMedia(data['media'])
+        if data.get('rel', None) is not None:
+            stylesheet.setRel(data['rel'])
+        if data.get('title', None) is not None:
+            stylesheet.setTitle(data['title'])
+        if data.get('rendering', None) is not None:
+            stylesheet.setRendering(data['rendering'])
+        if data.get('enabled', None) is not None:
+            stylesheet.setEnabled(data['enabled'])
+        if data.get('cookable', None) is not None:
+            stylesheet.setCookable(data['cookable'])
+        if data.get('compression', None) is not None:
+            stylesheet.setCompression(data['compression'])
+        if data.get('cacheable', None) is not None:
+            stylesheet.setCacheable(data['cacheable'])
+
     security.declareProtected(permissions.ManagePortal, 'getRenderingOptions')
     def getRenderingOptions(self):
         """Rendering methods for use in ZMI forms."""
         return config.CSS_RENDER_METHODS
 
+    security.declareProtected(permissions.ManagePortal, 'getCompressionOptions')
+    def getCompressionOptions(self):
+        """Compression methods for use in ZMI forms."""
+        return config.CSS_COMPRESSION_METHODS
+
     security.declareProtected(permissions.View, 'getContentType')
     def getContentType(self):
         """Return the registry content type."""
